<?php

/**
 * Gets the plugin specific option key
 */
function emojics_get_option_key( $option = null, $default = false ) {
	if ( is_null( $option ) ) return $default;

	return '_emojics_' . $option;
}

/**
 * Generic Option getter
 */
function emojics_get_option( $option = null, $default = false ) {
	if ( is_null( $option ) ) return $default;

	$option_value = get_option(emojics_get_option_key($option), $default);

	return $option_value;
}

/**
 * Generic Option setter
 */
function emojics_set_option( $option = null, $value ) {
	if ( is_null( $option ) ) return false;

	$result = update_option(emojics_get_option_key($option), $value);

	return $result;
}
/**
 * Generic Option delete
 */
function emojics_delete_option( $option = null ) {
	if ( is_null( $option ) ) return false;

	$result = delete_option(emojics_get_option_key($option));

	return $result;
}

/**
 * Get User IP
 *
 * Returns the IP address of the current visitor
 *
 */
function emojics_get_ip() {
	$ip = '127.0.0.1';
	if ( ! empty( $_SERVER['HTTP_CLIENT_IP'] ) ) {
		//check ip from share internet
		$ip = $_SERVER['HTTP_CLIENT_IP'];
	} elseif ( ! empty( $_SERVER['HTTP_X_FORWARDED_FOR'] ) ) {
		//to check ip is pass from proxy
		$ip = $_SERVER['HTTP_X_FORWARDED_FOR'];
	} elseif( ! empty( $_SERVER['REMOTE_ADDR'] ) ) {
		$ip = $_SERVER['REMOTE_ADDR'];
	}
	return apply_filters( 'emojics_get_ip', $ip );
}

/**
 * Returns the enviroment
 *
 * @return String
 * @since 1.0.0
 */
function emojics_get_env() {
	$user_ip = emojics_get_ip();

	$env = 'production';
	if ( substr($user_ip, 0, 4) == '127.' || $user_ip == '::1' ) {
		$env = 'development';
	}

	return apply_filters( 'emojics_get_enviroment', $env );
}

/**
 * Returns true if enviroment is development
 *
 * @return String
 * @since 1.0.0
 */
function emojics_is_env_dev() {

	$env = emojics_get_env();
	if ( $env == 'development' ) {
		return true;
	}

	return false;
}

/**
 * Returns true if enviroment is production
 *
 * @return String
 * @since 1.0.0
 */
function emojics_is_env_prod() {

	if ( ! emojics_is_env_dev() ) {
		return true;
	}

	return false;
}

/**
 * Returns an array with all the required registration fields
 *
 * @return Array
 * @since 1.2.0
 */
function emojics_get_registration_fields() {
	$user_info = wp_get_current_user();

	return [
		'first_name' => $user_info->user_firstname,
		'last_name'  => $user_info->user_lastname,
		'site'       => get_home_url(),
		'email'      => $user_info->user_email,
		'password'   => null
	];
}

/**
 * Returns an array of supported type positions based on the passed type
 *
 * @return Array
 * @since 1.2.0
 */
function emojics_get_initial_widget_configs() {
	$widget_configs = array();

	$excludes = (array)emojics_get_option('widget_exclude', array());
	$position = emojics_get_option('widget_position', 'floating');

	foreach (emojics_get_supported_types() as $key => $type) {
		$supported_positions = emojics_get_supported_type_position($key);

		$p = in_array($position, array_keys($supported_positions))?$position:'floating';

		switch ( $type->name ) {
			case 'post':
			case 'page':
				$p = 'inline';
				break;
		}

		$widget_configs[$key] = [
			'active'   => false,
			'position' => $p,
			'name'     => $type->name,
			'label'    => $type->label
		];
	}

	// $widget_configs['home']['active'] = (in_array('home', $excludes))?false:true;
	$widget_configs['post']['active'] = (in_array('post', $excludes))?false:true;
	$widget_configs['page']['active'] = (in_array('page', $excludes))?false:true;

	return $widget_configs;
}

/**
 * Returns an array of supported type positions based on the passed type
 *
 * @return Array
 * @since 1.2.0
 */
function emojics_get_supported_type_position($type = null) {

	$only_floating = [
		'floating' => 'Floating'
	];

	$only_inline = [
		'inline' => 'Inline (after content)'
	];

	$both = array_merge($only_floating, $only_inline);

	if ( !$type ) {
		return [];
	}

	switch ($type) {
		case 'home':
		case 'page_not_found':
			return $only_floating;
			break;

		case 'author':
		case 'page':
			return $both;
			break;
	}

	$types = emojics_get_supported_types();

	if ( isset($types[$type]) && $types[$type]->hierarchical ) {
		return $only_floating;
	}

	return $both;
}

/**
 * Returns all pages like post-types, taxonomies, author pages
 * where the widget is supported
 *
 * Note: types refers to all kind of pages, also taxonomies and author pages
 *
 * @return Array
 * @since 1.2.0
 */
function emojics_get_supported_types() {

	$unsupported = ['post_format'];

	$types = [
		'home' => (object)[
			'name'         => 'home',
			'label'        => 'Home Page',
			'hierarchical' => false
		]
	];

	$types = array_merge( $types, get_post_types(array(
		'public'   => true,
		// '_builtin' => true
	), 'objects'));

	$types = array_merge( $types, [
		'author' => (object)[
			'name'         => 'author',
			'label'        => 'Author Pages',
			'hierarchical' => true
		]
	]);

	$types = array_merge( $types, get_taxonomies(array(
		'public'   => true,
		// '_builtin' => true
	), 'objects'));

	$types = array_merge( $types, [
		'page_not_found' => (object)[
			'name'         => 'page_not_found',
			'label'        => '404 Pages',
			'hierarchical' => false
		]
	]);

	/**
	 * Remove unsupported types
	 */
	foreach ($unsupported as $key) {
		if ( isset($types[$key]) ) {
			unset($types[$key]);
		}
	}

	return apply_filters( 'emojics_get_post_types', $types);
}
